<?php

class lcso_htaccess_tweaks {
    
    const TWEAK_VERSION = '1.1'; // (string) HTACCESS tweak code versioning
    
    private $htaccess_path = ABSPATH .'/.htaccess';
    private $bkp_opt_name = 'lcso_htaccess_bkp'; // (string) backup option's name 
    

    
    /*
     * Get HTACCESS file contents
     * @return (false|string) the contents string or false if file does not exists or is not manageable via code
     */
    private function get_contents() {
        global $wp_filesystem;
        WP_Filesystem();
        
        $contents = $wp_filesystem->get_contents($this->htaccess_path);
        return (!$contents) ? false : $contents;
    }
    
    
    
    
    /* Given the HTACCESS contents string, returns them without the LCSO part */
    private function strip_lcso_code($str) {
        return preg_replace('/### LCSO BEGIN(.*?)LCSO END ###/s', '', $str);
    }
    
    
    
    
    /*
     * Tweak HTACCESS - set expire headers for static files
     * @return (true|string) true if everything was okay, otherwise the error string
     */
    public function tweak_it() {
        global $wp_filesystem;
        
        $contents = $this->get_contents();
        if(!$contents) {
            return esc_html__('HTACCESS file does not exist or cannot be managed', 'lcso_ml');
        }
        
        // create a safety backup
        if(!get_option($this->bkp_opt_name)) {
            update_option($this->bkp_opt_name, $contents, false);   
        }
        
        
        $to_append = 
'### LCSO BEGIN (v'. self::TWEAK_VERSION .') ###
<IfModule mod_expires.c>
    ExpiresActive On
    ExpiresByType image/jpg "access 1 year"
    ExpiresByType image/jpeg "access 1 year"
    ExpiresByType image/gif "access 1 year"
    ExpiresByType image/png "access 1 year"
    ExpiresByType image/svg "access 1 year"
    ExpiresByType image/webp "access 1 year"
    ExpiresByType image/avif "access 1 year"
    ExpiresByType image/x-icon "access 1 year"
    
    ExpiresByType application/x-font-ttf "access 1 year"
    ExpiresByType font/woff "access 1 year"
    ExpiresByType font/woff2 "access 1 year"
    
    ExpiresByType text/css "access 1 year"
    ExpiresByType application/pdf "access 1 year"
    ExpiresByType application/javascript "access 1 year"
    ExpiresByType application/x-javascript "access 1 year"
</IfModule>
### LCSO END ###';
        
        
        // tweak already applied? do nothing
        if(strpos($contents, '### LCSO BEGIN (v'. self::TWEAK_VERSION .') ###') !== false) {
            return true;   
        }
        
        $new_contents = $this->strip_lcso_code($contents) .'


'. $to_append;
        
        
        $response = $wp_filesystem->put_contents($this->htaccess_path, $new_contents, FS_CHMOD_FILE);
        return ($response) ? true : esc_html__('Error updating HTACCESS file contents', 'lcso_ml');
    }
    
    
    
    
    /*
     * Restore HTACCESS contents
     * @return (true|string) true if everything was okay, otherwise the error string
     */
    public function restore_it() {
        global $wp_filesystem;
        
        $contents = $this->get_contents();
        if(!$contents) {
            return esc_html__('HTACCESS file does not exist or cannot be managed', 'lcso_ml');
        }
        
        
        $clean_contents = $this->strip_lcso_code($contents);
        
        // no code changed - do not edit HTACCESS
        if($clean_contents == $contents) {
            return true;   
        }
        
        $response = $wp_filesystem->put_contents($this->htaccess_path, $clean_contents, FS_CHMOD_FILE);
        if($response) {
            delete_option($this->bkp_opt_name); 
            return true;
        }
        else {
            esc_html__('Error updating HTACCESS file contents', 'lcso_ml'); 
        }
    }   
}